@@ -24,10 +24,11 @@ from tempfile import TemporaryFile
 # import local interfaces
 from ztfy.thesaurus.interfaces.loader import IThesaurusLoader, IThesaurusLoaderHandler, \
                                              IThesaurusExporter, IThesaurusExporterHandler
+from ztfy.thesaurus.interfaces.term import IThesaurusLoaderTerm
 from ztfy.thesaurus.interfaces.thesaurus import IThesaurusInfoBase
 
 # import Zope3 packages
-from zope.interface import implements
+from zope.interface import implements, alsoProvides
 from zope.publisher.browser import FileUpload
 from zope.schema.fieldproperty import FieldProperty
 
@@ -67,19 +68,21 @@ class ThesaurusLoaderDescription(object):
 class ThesaurusLoaderTerm(object):
     """Base thesaurus loader term"""
 
-    def __init__(self, label, alt=None, definition=None, note=None, generic=None, specifics=[],
-                 associations=[], usage=None, used_for=[], created=None, modified=None):
+    def __init__(self, label, alt=None, definition=None, note=None, generic=None, specifics=None,
+                 associations=None, usage=None, used_for=None, created=None, modified=None, weight=0, properties=None):
         self.label = label
         self.alt = alt
         self.definition = definition
         self.note = note
         self.generic = generic
-        self.specifics = specifics
-        self.associations = associations
+        self.specifics = specifics or []
+        self.associations = associations or []
         self.usage = usage
-        self.used_for = used_for
+        self.used_for = used_for or []
         self.created = created
         self.modified = modified
+        self.weight = int(weight)
+        self.properties = properties or {}
 
 
 class BaseThesaurusLoaderHandler(object):
@@ -119,16 +122,42 @@ class BaseThesaurusLoader(object):
         for key, term in terms.iteritems():
             new_term = key_store[key]
             if term.generic:
-                new_term.generic = key_store.get(term.generic)
+                target = key_store.get(term.generic)
+                if target is None:
+                    target = ThesaurusTerm(label=term.generic)
+                    alsoProvides(target, IThesaurusLoaderTerm)
+                    key_store[target.label] = store[target.label] = target
+                new_term.generic = target
             if term.specifics:
+                for specific in term.specifics:
+                    if key_store.get(specific) is None:
+                        target = ThesaurusTerm(label=specific)
+                        alsoProvides(target, IThesaurusLoaderTerm)
+                        key_store[target.label] = store[target.label] = target
                 new_term.specifics = [key_store.get(specific) for specific in term.specifics]
                 for subterm in new_term.specifics:
                     subterm.generic = new_term
             if term.associations:
+                for association in term.associations:
+                    if key_store.get(association) is None:
+                        target = ThesaurusTerm(label=association)
+                        alsoProvides(target, IThesaurusLoaderTerm)
+                        key_store[target.label] = store[target.label] = target
                 new_term.associations = [key_store.get(association) for association in term.associations]
             if term.usage:
-                new_term.usage = key_store.get(term.usage)
+                target = key_store.get(term.usage)
+                if target is None:
+                    target = ThesaurusTerm(label=term.usage)
+                    alsoProvides(target, IThesaurusLoaderTerm)
+                    key_store[target.label] = store[target.label] = target
+                new_term.usage = target
+                target.used_for = set(target.used_for) | set((new_term,))
             if term.used_for:
+                for used in term.used_for:
+                    if key_store.get(used) is None:
+                        target = ThesaurusTerm(label=used)
+                        alsoProvides(target, IThesaurusLoaderTerm)
+                        key_store[target.label] = store[target.label] = target
                 new_term.used_for = [key_store.get(used) for used in term.used_for]
                 for synonym in new_term.used_for:
                     synonym.usage = new_term
